#//-----------------------------------------------
#//TUSB-K16DIOZ用ドライバアクセス用API
#//Python 用モジュール
#//
#//2024_04_08　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------

from ctypes import *

#Device DLL Loader ( cdecl )
tusbk16diodll = cdll.LoadLibrary("TUSBKDIO.DLL")

tusbk16diodll.TUSBK16DIO_Device_Open.restype = c_short
tusbk16diodll.TUSBK16DIO_Device_Open.argtypes = (c_short,)
tusbk16diodll.TUSBK16DIO_Device_Close.restype = None
tusbk16diodll.TUSBK16DIO_Device_Close.argtypes = (c_short,)
tusbk16diodll.TUSBK16DIO_Dir.restype = c_short
tusbk16diodll.TUSBK16DIO_Dir.argtypes = (c_short,c_ubyte,c_ubyte)
tusbk16diodll.TUSBK16DIO_PortWrite.restype = c_short
tusbk16diodll.TUSBK16DIO_PortWrite.argtypes = (c_short,c_ubyte,c_ubyte)
tusbk16diodll.TUSBK16DIO_Write.restype = c_short
tusbk16diodll.TUSBK16DIO_Write.argtypes = (c_short,c_ubyte,c_ubyte)
tusbk16diodll.TUSBK16DIO_Read.restype = c_short
tusbk16diodll.TUSBK16DIO_Read.argtypes = (c_short,POINTER(c_ubyte),POINTER(c_ubyte))
tusbk16diodll.TUSBK16DIO_DirRead.restype = c_short
tusbk16diodll.TUSBK16DIO_DirRead.argtypes = (c_short,POINTER(c_ubyte),POINTER(c_ubyte))
tusbk16diodll.TUSBK16DIO_OutputRead.restype = c_short
tusbk16diodll.TUSBK16DIO_OutputRead.argtypes = (c_short,POINTER(c_ubyte),POINTER(c_ubyte,))

#######  support functions (don not call directry from user codes)

def isint(n):
    return type(n) is int

def IsInRange(Val,Min,Max):
    if(isint(Val) == False ):   return False
    elif(Val < Min):            return False
    elif(Val > Max):            return False
    else:                       return True

def IsValidId(DevId):
    return IsInRange(DevId,0,15)

#def functions

def TUSBK16DIO_Device_Open(DevId):
    """Open Device Connection
    
    Arguments:
    DevId           ----    Device ID 0~15

    Returns:
    int             ----    Result code
    """
    if(not IsValidId(DevId)): return 8 # Invalid parameter
    return tusbk16diodll.TUSBK16DIO_Device_Open(DevId)
    
def TUSBK16DIO_Device_Close(DevId):
    """Close Device Connection
    
    Arguments:
    DevId           ----    Device ID 0~15
    """
    if(not IsValidId(DevId)): return # Invalid parameter
    tusbk16diodll.TUSBK16DIO_Device_Close(DevId)
    
def TUSBK16DIO_Dir(DevId,Port,Dir):
    """Set Port Directions
    
    Arguments:
    DevId           ----    Device ID 0~15
    Port            ----    Port Select 0:port1 1:port2
    Dir             ----    Port Direction 0:input 1:output

    Returns:
    int             ----    Result code
    """
    if(not IsValidId(DevId)): return 8 # Invalid parameter
    if(not IsInRange(Port,0,1)): return 8 # Invalid parameter
    if(not IsInRange(Dir,0,1)): return 8 # Invalid parameter
    return tusbk16diodll.TUSBK16DIO_Dir(DevId,Port,Dir)

def TUSBK16DIO_PortWrite(DevId,Port,Data):
    """Write Output Port Resister
    
    Arguments:
    DevId           ----    Device ID 0~15
    Port            ----    Port Select 0:port1 1:port2
    Data            ----    Output bit pattern
    
    Returns:
    int             ----    Result code
    """
    if(not IsValidId(DevId)): return 8 # Invalid parameter
    if(not IsInRange(Port,0,1)): return 8 # Invalid parameter
    if(not IsInRange(Data,0,255)): return 8 # Invalid parameter
    return tusbk16diodll.TUSBK16DIO_PortWrite(DevId,Port,Data)

def TUSBK16DIO_Write(DevId,Data1,Data2):
    """Write Output Resisters
    
    Arguments:
    DevId           ----    Device ID 0~15
    Data1           ----    Output data bits port1
    Data2           ----    Output data bits port2

    Returns;
    int             ----    Result code
    """
    if(not IsValidId(DevId)): return 8 # Invalid parameter
    if(not IsInRange(Data1,0,255)): return 8 # Invalid parameter
    if(not IsInRange(Data2,0,255)): return 8 # Invalid parameter
    return tusbk16diodll.TUSBK16DIO_Write(DevId,Data1,Data2)

def TUSBK16DIO_Read(DevId):
    """Read Input Resister
    
    Arguments:
    DevId           ----    Device ID 0~15
    
    Returns:
    int             ----    Result code
    int             ----    Input Data port1
    int             ----    Input Data port2
    """
    if(not IsValidId(DevId)): return 8 # Invalid parameter
    Data1 = c_ubyte()
    Data2 = c_ubyte()
    ret =  tusbk16diodll.TUSBK16DIO_Read(DevId,byref(Data1),byref(Data2))
    if ret == 0:
        return 0,Data1.value,Data2.value
    else:
        return ret,None,None

def TUSBK16DIO_DirRead(DevId):
    """Read Direction Resister
    
    Arguments:
    DevId           ----    Device ID 0~15
    
    Returns:
    int             ----    Result code
    int             ----    Port1 direction  0:input 1:output
    int             ----    Port1 direction  0:input 1:output
    """
    if(not IsValidId(DevId)): return 8 # Invalid parameter
    Dir1 = c_ubyte()
    Dir2 = c_ubyte()
    ret = tusbk16diodll.TUSBK16DIO_DirRead(DevId,byref(Dir1),byref(Dir2))
    if ret == 0:
        return 0,Dir1.value,Dir2.value
    else:
        return ret,None,None
    
def TUSBK16DIO_OutputRead(DevId):
    """Read Output port resisters
    
    Arguments:
    DevId           ----    Device ID 0~15
    
    Returns:
    int             ----    Result code
    int             ----    Output Data port1
    int             ----    Output Data port2
    """
    if(not IsValidId(DevId)): return 8 # Invalid parameter
    Data1 = c_ubyte()
    Data2 = c_ubyte()
    ret = tusbk16diodll.TUSBK16DIO_OutputRead(DevId,byref(Data1),byref(Data2))
    if ret == 0:
        return 0,Data1.value,Data2.value
    else:
        return ret,None,None


#戻り値用エラーメッセージ(オープン関数)
ErrStrDic = {
            0:"正常終了しました",
            1:"ID番号が不正です",
            2:"ドライバが初期化されていません",
            3:"すでにデバイスはオープンされています",
            4:"接続されている台数が多すぎます",
            5:"オープンできませんでした",
            6:"デバイスがみつかりません",
            8:"パラメータエラーです",
            9:"USB通信エラーです",
            99:"その他のエラーです"
}
def GetErrMessage(retcode):
    if( retcode in ErrStrDic):
        return ErrStrDic[retcode]
    else:
        return "不明なエラーです"

