#//-----------------------------------------------
#//TUSB-K16DIOZ用サンプルプログラム
#//Python 用モジュール
#//
#//2024_04_08　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------
import TUSBKDIO

from SpFuncs import *
from functools import partial



class Tusbkdio_Device:
    """TUSB-K16DIO Device Control"""

    DevId = None
    mon_t = None
    lock = threading.Lock() # デバイスコマンドが同時実行されないために使用するロック

    def __init__(self,DevId):
        """constructor
        
        arguments:
        DevId           ----    Device id 0~15
        """
        self.formgen()
        ret = TUSBKDIO.TUSBK16DIO_Device_Open(DevId)
        if ret == 0:
            self.DevId = DevId
            self.form.protocol("WM_DELETE_WINDOW", self.DevClose)   # フォームが閉じられる時に処理を行うため
            if self.DevInit():
                self.mon_t = intTT(0.5,self.Monitor)    # タイマースレッドを開始してタイマーからMonitorメソッドを呼び出させる
                self.mon_t.Start()
                return
            TUSBKDIO.TUSBK16DIO_Device_Close(DevId)
        ShowInfo(TUSBKDIO.GetErrMessage(ret))
        self.form.destroy()                             # 何か問題が有ったら終了とする
    
    def DevInit(self):
        """デバイスの状態を確認して画面表示に反映する"""
        # 入出力方向設定を表示に反映
        ret = TUSBKDIO.TUSBK16DIO_DirRead(self.DevId)
        if ret[0] != 0:
            ShowInfo(TUSBKDIO.GetErrMessage(ret[0]))
            return False
        self.RBtnDir[0].set(ret[1])
        self.RBtnDir[1].set(ret[2])
        # 出力ビット状態High Lowを読み出してチェックボックスに反映する
        ret = TUSBKDIO.TUSBK16DIO_OutputRead(self.DevId)
        if ret[0] != 0:
            ShowInfo(TUSBKDIO.GetErrMessage(ret[0]))
            return False
        for port in range(2):
            for bit in range(8):
                self.ChkBit[port][bit].set(IsBitSet(ret[port+1],bit))
        # ラジオボタンとチェックボックス変更時のコマンドハンドラを設定する
        for port in range(2):
            for dir in range(2):
                self.rbtn[port][dir].config(command=partial(self.DirSel_on,port=port))
            for bit in range(8):
                self.chkb[port][bit].config(command=partial(self.Check_on,port=port))
        return True
        
    def DevClose(self):
        """デバイスを閉じる時の処理"""
        if self.DevId != None:
            self.mon_t.Stop()
            TUSBKDIO.TUSBK16DIO_Device_Close(self.DevId)
            self.form.destroy()

    def DirSel_on(self,port):
        """入出力方向が変更された時のハンドラ"""
        dir = 1 if self.RBtnDir[port].get() == True else 0
        if self.lock.acquire(timeout=1):
            ret = TUSBKDIO.TUSBK16DIO_Dir(self.DevId,port,dir)
            self.lock.release()
            if ret != 0:
                ShowInfo(TUSBKDIO.GetErrMessage(ret))

    def Check_on(self,port):
        """出力ビットが変更された時のハンドラ"""
        data = 0
        for bit in range(8):
            data += (1<<bit) if self.ChkBit[port][bit].get() else 0
        if self.lock.acquire(timeout=1):
            ret = TUSBKDIO.TUSBK16DIO_PortWrite(self.DevId,port,data)
            self.lock.release()
            if ret != 0:
                ShowInfo(TUSBKDIO.GetErrMessage(ret))
    
    # timer -------------------------------------
    def Monitor(self):
        """入力ビット確認処理 タイマーから呼び出される"""
        if self.DevId != None:
            if self.lock.acquire(timeout=1):
                ret = TUSBKDIO.TUSBK16DIO_Read(self.DevId)
                self.lock.release()
                if ret[0] == 0:
                    for port in range(2):
                        for bit in range(8):
                            if IsBitSet(ret[port+1],bit): self.StatusDisp[port][bit]['text'] = "High"
                            else: self.StatusDisp[port][bit]['text'] = "Low"

    # form generation ---------------------------
    def formgen(self):
        self.form = tk.Tk()
        self.form.geometry('328x255')
        self.form.title('TUSB-K16DIO Sample Program')

        frame = GenList(2,None)                         # port1 port2 フレーム(枠)
        self.chkb = GenList2D(2,8,None)                      # チェックボックス
        self.rbtn = GenList(2,None)                          # ラジオボタン
        self.StatusDisp = GenList2D(2,8,None)                # 入力状態表示ラベル
        self.ChkBit = GenTkVarList2D(2,8,tk.BooleanVar,False)# チェックボックスのチェック状態
        self.RBtnDir = GenTkVarList(2,tk.IntVar,0)           # ラジオボタンの選択状態(値)

        for port in range(2):# ポート毎にウィジェット作成
            # フレーム
            frame[port] = GetLFrame(self.form,"Port"+str(port+1),0.46,0.9,0.03+port*0.48,0.05)
            # ラジオボタン
            FilterDict= {
                "Input":0,
                "Output":1,
            }
            self.rbtn[port] = GetRadioButtons(frame[port],FilterDict,0.05,0.07,0.45,0,self.RBtnDir[port])
            # 表示ラベル
            GetLabel(frame[port],"Output",0.15,0.17)
            GetLabel(frame[port],"Status",0.6,0.17)
            # チェックボックスと入力表示
            for bit in range(8):
                self.chkb[port][bit] = GetChkBox(frame[port],str(bit),0.18,0.3+0.09*bit,self.ChkBit[port][bit])
                self.StatusDisp[port][bit] = GetLabelDisp(frame[port],"",0.3,0.55,0.3+0.09*bit)

# テスト用 ---------------------------------
if __name__=="__main__":
    tmp = Tusbkdio_Device(DevId=0)
    tmp.form.mainloop()



